﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using XIPlugin;
using System.Windows.Forms;
using NLog;

namespace $safeprojectname$
{
    public class XIPlugin:XIPluginInterface,XIMultiInstancePlugin
    {
        /*
         * Creates a NLog logger for this class.  Try to use the correct
         * logging level when logging information (from lowest to highest):
         * Trace: The lowest level, use for fine detail.  i.e. entering/exiting logging
         * Debug: Information on the application flow.  Use for debugging info during
         * development.  This is the lowest level that appears in the console by default.
         * Info: Used for interesting runtime events.  Starting/stopped logging should go here
         * Warn: Used for runtime conditions that are bad, but not necessarily a dealbreaker.  
         * This is the lowest level that is logged to a file by default.
         * Error: Pretty obvious, these are serious problems.
         * 
         * The inclusion of NLog is to encourage thorough logging.  Ideally, all a user should
         * need to do in order to provide precise logs in the event of an error is to change to the 
         * appropriate logging level and send in their text log.
         */
        private static Logger logger = LogManager.GetCurrentClassLogger();

        /*
         * SettingsControl object, need to hang onto a pointer in order
         * to properly handle save calls
         */
        private SettingsControl settingsControl;

        /*
         * This dictionary holds on to the ffxi instance list, as a dictionary 
         * with the format:
         * [processId => FFXIWindow]
         */
        private Dictionary<int, FFXIWindow> windows;


        /*
         * The startPlugin call is used to setup any necessary data or 
         * callbacks for the plugin.  This is where you would setup
         * low level hardware callbacks for plugins with message support, or
         * initialize any fface/windower text fields
         */
        public void startPlugin(Form mainApp)
        {
            logger.Debug("Starting simple plugin");
        }

        /*
         * Plugin name.
         */
        public string Name
        {
            get
            {
                return "Simple";
            }
        }

        /*
         * This method is called in different contexts depending on the
         * plugin type.  For a multi instance plugin, this is called every time a 
         * ffxi window is created or exits, along with on creation.
         * For a single instance plugin, this is only called at creation.
         */
        public void setPluginContext(XIPluginContext context)
        {
            logger.Trace("Plugin Context updated");
            windows = context.windows;
        }

        /*
         * This method is called when the plugin needs to terminate.  
         * If the plugin thread is still active some time after calling this, 
         * it will be force stopped.  Clean up any windower or fface
         * text boxes here, take care of any timers, etc.
         */
        public void stopPlugin()
        {

        }

        /*
         * This returns the keyboard command associated with this plugin.  Any 
         * windower input matching this command will be sent to the processCommand 
         * method in this plugin.  If no keyboard command is necessary for a plugin,
         * return null from this method.
         */
        public string Command
        {
            get
            {
                return "simple";
            }
        }

        /*
         * This method is called when a matching keyboard command is found.  
         * For example, the command for this plugin is "simple", typing
         * "//simple test 1 2 3" into the chatline will call this method with a
         * XICommandContext object with args of ["test", "1", "2", "3"]
         * It is important to note that processCommand is called in a new
         * thread, so be sure to use threadsafe access to data
         */
        public void processCommand(object args)
        {
            XICommandContext xic = (XICommandContext)args;
            List<String> commandArgs = xic.args;

            logger.Debug("CommandArgs from pid "+xic.pid+": ");
            foreach (String str in commandArgs)
            {
                logger.Debug(str);
            }

        }

        /*
         * Plugin description, this shows up in the plugin selection box
         */
        public string Description
        {
            get
            {
                return "Simple Debugging Plugin";
            }
        }

        /*
         * This should return the display tab of the plugin, if one is needed.
         * For plugins that do not require a display tab, return null.
         */
        public TabPage DisplayTab
        {
            get
            {
                TabPage t = new TabPage(Name);
                t.Controls.Add(new TabControl());
                return t;
            }
        }

        /*
         * This should return the settings tab of the plugin, if one is needed.
         * For plugins that do not require a settings tab, return null.
         */
        public TabPage SettingsTab
        {
            get
            {
                if (settingsControl == null || settingsControl.IsDisposed)
                {
                    settingsControl = new SettingsControl();
                }
                TabPage t = new TabPage(Name);
                t.Controls.Add(settingsControl);
                return t;
            }
        }

        /*
         * This method is called when settings should be saved.  It is expected that
         * the plugin will read in settings from the SettingsTab, and save them as necessary.
         */
        public void saveSettings()
        {
            SettingsModel settings = SettingsModel.getInstance();

            settings.someCheckbox = settingsControl.someCheckbox.Checked;

            logger.Debug("Saving settings");
            settings.SaveSettings();
        }

    }
}
